﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic.Regression {
  [Item("ReplaceBranchMove", "")]
  [StorableClass]
  public class ReplaceBranchMove : Item {
    [Storable]
    public int FragmentIndex { get; set; }
    [Storable]
    public ISymbolicExpressionTree Tree { get; set; }

    [Storable]
    public ISymbolicExpressionTreeNode Parent { get; set; }

    [Storable]
    public int SubtreeIndex { get; set; }

    [Storable]
    public ISymbolicExpressionTreeNode NewBranch { get; set; }

    [Storable]
    public double[] OriginalOutput { get; private set; }

    [Storable]
    public double[] NewOutput { get; private set; }

    [Storable]
    public double Alpha { get; set; }

    [Storable]
    public double Beta { get; set; }


    [StorableConstructor]
    protected ReplaceBranchMove(bool deserializing) : base(deserializing) { }
    protected ReplaceBranchMove(ReplaceBranchMove original, Cloner cloner)
      : base(original, cloner) {
      this.FragmentIndex = original.FragmentIndex;
      this.Tree = cloner.Clone(original.Tree);
      this.SubtreeIndex = original.SubtreeIndex;
      this.Parent = cloner.Clone(original.Parent);
      this.NewBranch = cloner.Clone(original.NewBranch);
      this.OriginalOutput = original.OriginalOutput;
      this.NewOutput = original.NewOutput;
      this.Alpha = original.Alpha;
      this.Beta = original.Beta;
    }
    public ReplaceBranchMove(ISymbolicExpressionTree tree, ISymbolicExpressionTreeNode parent, int subtreeIndex, ISymbolicExpressionTreeNode newChild, double[] originalOutput, double[] newOutput, int fragmentIndex)
      : base() {
      this.Tree = tree;
      this.SubtreeIndex = subtreeIndex;
      this.Parent = parent;
      this.NewBranch = newChild;
      this.OriginalOutput = originalOutput;
      this.NewOutput = newOutput;
      this.FragmentIndex = fragmentIndex;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ReplaceBranchMove(this, cloner);
    }
  }
}
